@@ -1,13 +1,15 @@
+import asyncio
 import time
 
 import aiohttp
 from aiohttp import ClientSession
 
-from pymax.files import Photo
+from pymax.files import File, Photo
 from pymax.formatting import Formatting
 from pymax.interfaces import ClientProtocol
 from pymax.payloads import (
     AddReactionPayload,
+    AttachFilePayload,
     AttachPhotoPayload,
     DeleteMessagePayload,
     EditMessagePayload,
@@ -22,8 +24,9 @@ from pymax.payloads import (
     ReplyLink,
     SendMessagePayload,
     SendMessagePayloadMessage,
-    UploadPhotoPayload,
+    UploadPayload,
 )
+from pymax.static.constant import DEFAULT_TIMEOUT
 from pymax.static.enum import AttachType, Opcode
 from pymax.types import (
     Attach,
@@ -35,10 +38,70 @@ from pymax.types import (
 
 
 class MessageMixin(ClientProtocol):
+    async def _upload_file(self, file: File) -> None | Attach:
+        try:
+            self.logger.info("Uploading file")
+            payload = UploadPayload().model_dump(by_alias=True)
+            data = await self._send_and_wait(
+                opcode=Opcode.FILE_UPLOAD,
+                payload=payload,
+            )
+            if error := data.get("payload", {}).get("error"):
+                self.logger.error("Upload file error: %s", error)
+                return None
+
+            url = data.get("payload", {}).get("info", [None])[0].get("url", None)
+            file_id = data.get("payload", {}).get("info", [None])[0].get("fileId", None)
+            if not url or not file_id:
+                self.logger.error("No upload URL or file ID received")
+                return None
+
+            file_bytes = await file.read()
+
+            headers = {
+                "Content-Disposition": f"attachment; filename={file.file_name}",
+                "Content-Range": f"0-{len(file_bytes) - 1}/{len(file_bytes)}",
+            }
+
+            loop = asyncio.get_running_loop()
+            fut: asyncio.Future[dict] = loop.create_future()
+            try:
+                self._file_upload_waiters[int(file_id)] = fut
+            except Exception:
+                self.logger.exception("Failed to register file upload waiter")
+
+            async with (
+                ClientSession() as session,
+                session.post(
+                    url=url,
+                    headers=headers,
+                    data=file_bytes,
+                ) as response,
+            ):
+                if response.status != 200:
+                    self.logger.error(f"Upload failed with status {response.status}")
+                    # cleanup waiter
+                    self._file_upload_waiters.pop(int(file_id), None)
+                    return None
+
+                try:
+                    await asyncio.wait_for(fut, timeout=DEFAULT_TIMEOUT)
+                    return Attach(_type=AttachType.FILE, file_id=file_id)
+                except asyncio.TimeoutError:
+                    self.logger.error(
+                        "Timed out waiting for file processing notification for fileId=%s",
+                        file_id,
+                    )
+                    self._file_upload_waiters.pop(int(file_id), None)
+                    return None
+        except Exception as e:
+            self.logger.exception("Upload file failed: %s", str(e))
+            return None
+
     async def _upload_photo(self, photo: Photo) -> None | Attach:
         try:
             self.logger.info("Uploading photo")
-            payload = UploadPhotoPayload().model_dump(by_alias=True)
+            payload = UploadPayload().model_dump(by_alias=True)
 
             data = await self._send_and_wait(
                 opcode=Opcode.PHOTO_UPLOAD,
@@ -74,9 +137,7 @@ class MessageMixin(ClientProtocol):
                 ) as response,
             ):
                 if response.status != 200:
-                    self.logger.error(
-                        f"Upload failed with status {response.status}"
-                    )
+                    self.logger.error(f"Upload failed with status {response.status}")
                     return None
 
                 result = await response.json()
@@ -99,52 +160,61 @@ class MessageMixin(ClientProtocol):
             self.logger.exception("Upload photo failed: %s", str(e))
             return None
 
+    async def _upload_attachment(self, attach: Photo | File) -> dict | None:
+        if isinstance(attach, Photo):
+            uploaded = await self._upload_photo(attach)
+            if uploaded and uploaded.photo_token:
+                return AttachPhotoPayload(photo_token=uploaded.photo_token).model_dump(
+                    by_alias=True
+                )
+        elif isinstance(attach, File):
+            uploaded = await self._upload_file(attach)
+            if uploaded and uploaded.file_id:
+                return AttachFilePayload(file_id=uploaded.file_id).model_dump(
+                    by_alias=True
+                )
+        self.logger.error(f"Attachment upload failed for {attach}")
+        return None
+
     async def send_message(
         self,
         text: str,
         chat_id: int,
         notify: bool,
-        photo: Photo | None = None,
-        photos: list[Photo] | None = None,
+        attachment: Photo | File | None = None,
+        attachments: list[Photo | File] | None = None,
         reply_to: int | None = None,
+        use_queue: bool = False,
     ) -> Message | None:
         """
         Отправляет сообщение в чат.
         """
         try:
-            self.logger.info(
-                "Sending message to chat_id=%s notify=%s", chat_id, notify
-            )
-            if photos and photo:
-                self.logger.warning(
-                    "Both photo and photos provided; using photos"
-                )
-                photo = None
+            self.logger.info("Sending message to chat_id=%s notify=%s", chat_id, notify)
+            if attachments and attachment:
+                self.logger.warning("Both photo and photos provided; using photos")
+                attachment = None
             attaches = []
-            if photo:
-                self.logger.info("Uploading photo for message")
-                attach = await self._upload_photo(photo)
-                if not attach or not attach.photo_token:
-                    self.logger.error("Photo upload failed, message not sent")
+            if attachment:
+                self.logger.info("Uploading attachment for message")
+                result = await self._upload_attachment(attachment)
+                if not result:
+                    self.logger.error("Attachment upload failed, message not sent")
                     return None
-                attaches = [
-                    AttachPhotoPayload(
-                        photo_token=attach.photo_token
-                    ).model_dump(by_alias=True)
-                ]
-            elif photos:
-                self.logger.info("Uploading multiple photos for message")
-                for p in photos:
-                    attach = await self._upload_photo(p)
-                    if attach and attach.photo_token:
-                        attaches.append(
-                            AttachPhotoPayload(
-                                photo_token=attach.photo_token
-                            ).model_dump(by_alias=True)
-                        )
+                attaches.append(result)
+
+            elif attachments:
+                self.logger.info("Uploading multiple attachments for message")
+                for p in attachments:
+                    result = await self._upload_attachment(p)
+                    if result:
+                        attaches.append(result)
+                    else:
+                        self.logger.error("One of attachments upload failed")
+
                 if not attaches:
                     self.logger.error(
-                        "All photo uploads failed, message not sent"
+                        "All attachments uploads failed, message not sent"
                     )
                     return None
 
@@ -165,28 +235,27 @@ class MessageMixin(ClientProtocol):
                     cid=int(time.time() * 1000),
                     elements=elements,
                     attaches=attaches,
-                    link=(
-                        ReplyLink(message_id=str(reply_to))
-                        if reply_to
-                        else None
-                    ),
+                    link=(ReplyLink(message_id=str(reply_to)) if reply_to else None),
                 ),
                 notify=notify,
             ).model_dump(by_alias=True)
 
-            data = await self._send_and_wait(
-                opcode=Opcode.MSG_SEND, payload=payload
-            )
-            if error := data.get("payload", {}).get("error"):
-                self.logger.error("Send message error: %s", error)
+            if use_queue:
+                await self._queue_message(opcode=Opcode.MSG_SEND, payload=payload)
+                self.logger.debug("Message queued for sending")
                 return None
-            msg = (
-                Message.from_dict(data["payload"])
-                if data.get("payload")
-                else None
-            )
-            self.logger.debug("send_message result: %r", msg)
-            return msg
+            else:
+                data = await self._send_and_wait(
+                    opcode=Opcode.MSG_SEND, payload=payload
+                )
+                if error := data.get("payload", {}).get("error"):
+                    self.logger.error("Send message error: %s", error)
+                    return None
+                msg = (
+                    Message.from_dict(data["payload"]) if data.get("payload") else None
+                )
+                self.logger.debug("send_message result: %r", msg)
+                return msg
         except Exception:
             self.logger.exception("Send message failed")
             return None
@@ -196,47 +265,39 @@ class MessageMixin(ClientProtocol):
         chat_id: int,
         message_id: int,
         text: str,
-        photo: Photo | None = None,
-        photos: list[Photo] | None = None,
+        attachment: Photo | None = None,
+        attachments: list[Photo] | None = None,
+        use_queue: bool = False,
     ) -> Message | None:
-        """
-        Редактирует сообщение.
-        """
         try:
             self.logger.info(
                 "Editing message chat_id=%s message_id=%s", chat_id, message_id
             )
 
-            if photos and photo:
-                self.logger.warning(
-                    "Both photo and photos provided; using photos"
-                )
-                photo = None
+            if attachments and attachment:
+                self.logger.warning("Both photo and photos provided; using photos")
+                attachment = None
             attaches = []
-            if photo:
-                self.logger.info("Uploading photo for message")
-                attach = await self._upload_photo(photo)
-                if not attach or not attach.photo_token:
-                    self.logger.error("Photo upload failed, message not sent")
+            if attachment:
+                self.logger.info("Uploading attachment for message")
+                result = await self._upload_attachment(attachment)
+                if not result:
+                    self.logger.error("Attachment upload failed, message not sent")
                     return None
-                attaches = [
-                    AttachPhotoPayload(
-                        photo_token=attach.photo_token
-                    ).model_dump(by_alias=True)
-                ]
-            elif photos:
-                self.logger.info("Uploading multiple photos for message")
-                for p in photos:
-                    attach = await self._upload_photo(p)
-                    if attach and attach.photo_token:
-                        attaches.append(
-                            AttachPhotoPayload(
-                                photo_token=attach.photo_token
-                            ).model_dump(by_alias=True)
-                        )
+                attaches.append(result)
+
+            elif attachments:
+                self.logger.info("Uploading multiple attachments for message")
+                for p in attachment:
+                    result = await self._upload_attachment(p)
+                    if result:
+                        attaches.append(result)
+                    else:
+                        self.logger.error("One of attachments upload failed")
+
                 if not attaches:
                     self.logger.error(
-                        "All photo uploads failed, message not sent"
+                        "All attachments uploads failed, message not sent"
                     )
                     return None
 
@@ -257,24 +318,32 @@ class MessageMixin(ClientProtocol):
                 elements=elements,
                 attaches=attaches,
             ).model_dump(by_alias=True)
-            data = await self._send_and_wait(
-                opcode=Opcode.MSG_EDIT, payload=payload
-            )
-            if error := data.get("payload", {}).get("error"):
-                self.logger.error("Edit message error: %s", error)
-            msg = (
-                Message.from_dict(data["payload"])
-                if data.get("payload")
-                else None
-            )
-            self.logger.debug("edit_message result: %r", msg)
-            return msg
+
+            if use_queue:
+                await self._queue_message(opcode=Opcode.MSG_EDIT, payload=payload)
+                self.logger.debug("Edit message queued for sending")
+                return None
+            else:
+                data = await self._send_and_wait(
+                    opcode=Opcode.MSG_EDIT, payload=payload
+                )
+                if error := data.get("payload", {}).get("error"):
+                    self.logger.error("Edit message error: %s", error)
+                msg = (
+                    Message.from_dict(data["payload"]) if data.get("payload") else None
+                )
+                self.logger.debug("edit_message result: %r", msg)
+                return msg
         except Exception:
             self.logger.exception("Edit message failed")
             return None
 
     async def delete_message(
-        self, chat_id: int, message_ids: list[int], for_me: bool
+        self,
+        chat_id: int,
+        message_ids: list[int],
+        for_me: bool,
+        use_queue: bool = False,
     ) -> bool:
         """
         Удаляет сообщения.
@@ -291,14 +360,19 @@ class MessageMixin(ClientProtocol):
                 chat_id=chat_id, message_ids=message_ids, for_me=for_me
             ).model_dump(by_alias=True)
 
-            data = await self._send_and_wait(
-                opcode=Opcode.MSG_DELETE, payload=payload
-            )
-            if error := data.get("payload", {}).get("error"):
-                self.logger.error("Delete message error: %s", error)
-                return False
-            self.logger.debug("delete_message success")
-            return True
+            if use_queue:
+                await self._queue_message(opcode=Opcode.MSG_DELETE, payload=payload)
+                self.logger.debug("Delete message queued for sending")
+                return True
+            else:
+                data = await self._send_and_wait(
+                    opcode=Opcode.MSG_DELETE, payload=payload
+                )
+                if error := data.get("payload", {}).get("error"):
+                    self.logger.error("Delete message error: %s", error)
+                    return False
+                self.logger.debug("delete_message success")
+                return True
         except Exception:
             self.logger.exception("Delete message failed")
             return False
@@ -324,9 +398,7 @@ class MessageMixin(ClientProtocol):
                 pin_message_id=message_id,
             ).model_dump(by_alias=True)
 
-            data = await self._send_and_wait(
-                opcode=Opcode.CHAT_UPDATE, payload=payload
-            )
+            data = await self._send_and_wait(opcode=Opcode.CHAT_UPDATE, payload=payload)
             if error := data.get("payload", {}).get("error"):
                 self.logger.error("Pin message error: %s", error)
                 return False
@@ -376,8 +448,7 @@ class MessageMixin(ClientProtocol):
                 return None
 
             messages = [
-                Message.from_dict(msg)
-                for msg in data["payload"].get("messages", [])
+                Message.from_dict(msg) for msg in data["payload"].get("messages", [])
             ]
             self.logger.debug("History fetched: %d messages", len(messages))
             return messages
@@ -405,9 +476,7 @@ class MessageMixin(ClientProtocol):
             url (str): Ссылка на видео
         """
         try:
-            self.logger.info(
-                "Getting video_id=%s message_id=%s", video_id, message_id
-            )
+            self.logger.info("Getting video_id=%s message_id=%s", video_id, message_id)
 
             if self.is_connected and self._socket is not None:
                 payload = GetVideoPayload(
@@ -420,18 +489,14 @@ class MessageMixin(ClientProtocol):
                     video_id=video_id,
                 ).model_dump(by_alias=True)
 
-            data = await self._send_and_wait(
-                opcode=Opcode.VIDEO_PLAY, payload=payload
-            )
+            data = await self._send_and_wait(opcode=Opcode.VIDEO_PLAY, payload=payload)
 
             if error := data.get("payload", {}).get("error"):
                 self.logger.error("Get video error: %s", error)
                 return None
 
             video = (
-                VideoRequest.from_dict(data["payload"])
-                if data.get("payload")
-                else None
+                VideoRequest.from_dict(data["payload"]) if data.get("payload") else None
             )
             self.logger.debug("result: %r", video)
             return video
@@ -458,9 +523,7 @@ class MessageMixin(ClientProtocol):
             url (str): Ссылка на скачивание файла
         """
         try:
-            self.logger.info(
-                "Getting file_id=%s message_id=%s", file_id, message_id
-            )
+            self.logger.info("Getting file_id=%s message_id=%s", file_id, message_id)
             if self.is_connected and self._socket is not None:
                 payload = GetFilePayload(
                     chat_id=chat_id, message_id=message_id, file_id=file_id
@@ -480,9 +543,7 @@ class MessageMixin(ClientProtocol):
                 return None
 
             file = (
-                FileRequest.from_dict(data["payload"])
-                if data.get("payload")
-                else None
+                FileRequest.from_dict(data["payload"]) if data.get("payload") else None
             )
             self.logger.debug(" result: %r", file)
             return file
