@@ -2,6 +2,7 @@ import asyncio
 import socket
 import ssl
 import sys
+import time
 from collections.abc import Callable
 from typing import Any
 
@@ -116,8 +117,10 @@ Socket connections may be unstable, SSL issues are possible.
             self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
             self.is_connected = True
             self._incoming = asyncio.Queue()
+            self._outgoing = asyncio.Queue()
             self._pending = {}
             self._recv_task = asyncio.create_task(self._recv_loop())
+            self._outgoing_task = asyncio.create_task(self._outgoing_loop())
             self.logger.info("Socket connected, starting handshake")
             return await self._handshake(user_agent)
         except Exception as e:
@@ -431,6 +434,96 @@ Socket connections may be unstable, SSL issues are possible.
         finally:
             self._pending.pop(msg["seq"], None)
 
+    async def _outgoing_loop(self) -> None:
+        while self.is_connected:
+            try:
+                if self._outgoing is None:
+                    await asyncio.sleep(0.1)
+                    continue
+                
+                if self._circuit_breaker:
+                    if time.time() - self._last_error_time > 60:
+                        self._circuit_breaker = False
+                        self._error_count = 0
+                        self.logger.info("Circuit breaker reset (socket)")
+                    else:
+                        await asyncio.sleep(5)
+                        continue
+                
+                message = await self._outgoing.get() # TODO: persistent msg q mb?
+                if not message:
+                    continue
+                
+                retry_count = message.get("retry_count", 0)
+                max_retries = message.get("max_retries", 3)
+                
+                try:
+                    await self._send_and_wait(
+                        opcode=message["opcode"],
+                        payload=message["payload"],
+                        cmd=message.get("cmd", 0),
+                        timeout=message.get("timeout", 10.0)
+                    )
+                    self.logger.debug("Message sent successfully from queue (socket)")
+                    self._error_count = max(0, self._error_count - 1)
+                except Exception as e:
+                    self._error_count += 1
+                    self._last_error_time = time.time()
+                    
+                    if self._error_count > 10: # TODO: export to constant
+                        self._circuit_breaker = True
+                        self.logger.warning("Circuit breaker activated due to %d consecutive errors (socket)", self._error_count)
+                        await self._outgoing.put(message)
+                        continue
+                    
+                    retry_delay = self._get_retry_delay(e, retry_count)
+                    self.logger.warning("Failed to send message from queue (socket): %s (delay: %ds)", e, retry_delay)
+                    
+                    if retry_count < max_retries:
+                        message["retry_count"] = retry_count + 1
+                        await asyncio.sleep(retry_delay)
+                        await self._outgoing.put(message)
+                    else:
+                        self.logger.error("Message failed after %d retries, dropping (socket)", max_retries)
+                        
+            except Exception:
+                self.logger.exception("Error in outgoing loop (socket)")
+                await asyncio.sleep(1)
+
+    def _get_retry_delay(self, error: Exception, retry_count: int) -> float: # TODO: tune delays later
+        if isinstance(error, (ConnectionError, OSError, ssl.SSLError)):
+            return 1.0
+        elif isinstance(error, TimeoutError):
+            return 5.0
+        elif isinstance(error, SocketNotConnectedError):
+            return 2.0
+        else:
+            return 2 ** retry_count
+
+    async def _queue_message(
+        self,
+        opcode: int,
+        payload: dict[str, Any],
+        cmd: int = 0,
+        timeout: float = 10.0,
+        max_retries: int = 3,
+    ) -> None:
+        if self._outgoing is None:
+            self.logger.warning("Outgoing queue not initialized (socket)")
+            return
+            
+        message = {
+            "opcode": opcode,
+            "payload": payload,
+            "cmd": cmd,
+            "timeout": timeout,
+            "retry_count": 0,
+            "max_retries": max_retries,
+        }
+        
+        await self._outgoing.put(message)
+        self.logger.debug("Message queued for sending (socket)")
+
     async def _sync(self) -> None:
         try:
             self.logger.info("Starting initial sync (socket)")
