@@ -1,8 +1,19 @@
-from typing import Any, Literal
+from typing import Any, Final, Literal
 
-from pydantic import BaseModel, Field
+from pydantic import AliasChoices, BaseModel, Field
 
-from pymax.static import AttachType, AuthType
+from pymax.static.constant import (
+    DEFAULT_APP_VERSION,
+    DEFAULT_DEVICE_LOCALE,
+    DEFAULT_DEVICE_NAME,
+    DEFAULT_DEVICE_TYPE,
+    DEFAULT_LOCALE,
+    DEFAULT_OS_VERSION,
+    DEFAULT_SCREEN,
+    DEFAULT_TIMEZONE,
+    DEFAULT_USER_AGENT,
+)
+from pymax.static.enum import AttachType, AuthType
 
 
 def to_camel(string: str) -> str:
@@ -14,11 +25,12 @@ class CamelModel(BaseModel):
     model_config = {
         "alias_generator": to_camel,
         "populate_by_name": True,
+        "arbitrary_types_allowed": True,
     }
 
 
 class BaseWebSocketMessage(BaseModel):
-    ver: int = 11
+    ver: Final[int] = 11
     cmd: int
     seq: int
     opcode: int
@@ -52,20 +64,31 @@ class ReplyLink(CamelModel):
     message_id: str
 
 
-class UploadPhotoPayload(CamelModel):
+class UploadPayload(CamelModel):
     count: int = 1
 
 
 class AttachPhotoPayload(CamelModel):
-    type: AttachType = Field(AttachType.PHOTO, alias="_type")
+    type: AttachType = Field(default=AttachType.PHOTO, alias="_type")
     photo_token: str
 
 
+class AttachFilePayload(CamelModel):
+    type: AttachType = Field(default=AttachType.FILE, alias="_type")
+    file_id: int
+
+
+class MessageElement(CamelModel):
+    type: str
+    from_: int = Field(..., alias="from")
+    length: int
+
+
 class SendMessagePayloadMessage(CamelModel):
     text: str
     cid: int
-    elements: list[Any]
-    attaches: list[dict[str, Any]]
+    elements: list[MessageElement]
+    attaches: list[AttachPhotoPayload | AttachFilePayload]
     link: ReplyLink | None = None
 
 
@@ -79,8 +102,8 @@ class EditMessagePayload(CamelModel):
     chat_id: int
     message_id: int
     text: str
-    elements: list[Any]
-    attaches: list[Any]
+    elements: list[MessageElement]
+    attaches: list[AttachPhotoPayload]
 
 
 class DeleteMessagePayload(CamelModel):
@@ -95,7 +118,10 @@ class FetchContactsPayload(CamelModel):
 
 class FetchHistoryPayload(CamelModel):
     chat_id: int
-    from_time: int = Field(alias="from")
+    from_time: int = Field(
+        validation_alias=AliasChoices("from_time", "from"),
+        serialization_alias="from",
+    )
     forward: int
     backward: int = 200
     get_messages: bool = True
@@ -169,12 +195,18 @@ class ChangeGroupProfilePayload(CamelModel):
 
 
 class GetGroupMembersPayload(CamelModel):
-    type: str = "MEMBER"
+    type: Final[str] = "MEMBER"
     marker: int
     chat_id: int
     count: int
 
 
+class SearchGroupMembersPayload(CamelModel):
+    type: Final[str] = "MEMBER"
+    query: str
+    chat_id: int
+
+
 class NavigationEventParams(BaseModel):
     action_id: int
     screen_to: int
@@ -213,3 +245,48 @@ class SearchByPhonePayload(CamelModel):
 
 class JoinGroupPayload(CamelModel):
     link: str
+
+
+class ReactionInfoPayload(CamelModel):
+    reaction_type: str = "EMOJI"
+    id: str
+
+
+class AddReactionPayload(CamelModel):
+    chat_id: int
+    message_id: str
+    reaction: ReactionInfoPayload
+
+
+class GetReactionsPayload(CamelModel):
+    chat_id: int
+    message_ids: list[str]
+
+
+class RemoveReactionPayload(CamelModel):
+    chat_id: int
+    message_id: str
+
+
+class UserAgentPayload(CamelModel):
+    device_type: str = Field(default=DEFAULT_DEVICE_TYPE)
+    locale: str = Field(default=DEFAULT_LOCALE)
+    device_locale: str = Field(default=DEFAULT_DEVICE_LOCALE)
+    os_version: str = Field(default=DEFAULT_OS_VERSION)
+    device_name: str = Field(default=DEFAULT_DEVICE_NAME)
+    header_user_agent: str = Field(default=DEFAULT_USER_AGENT)
+    app_version: str = Field(default=DEFAULT_APP_VERSION)
+    screen: str = Field(default=DEFAULT_SCREEN)
+    timezone: str = Field(default=DEFAULT_TIMEZONE)
+
+
+class ReworkInviteLinkPayload(CamelModel):
+    revoke_private_link: bool = True
+    chat_id: int
+
+
+class RegisterPayload(CamelModel):
+    last_name: str | None = None
+    first_name: str
+    token: str
+    token_type: AuthType = AuthType.REGISTER
